# vim: et ts=4 sw=4
# Copyright © 2021 Antonio Terceiro <terceiro@debian.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from dhpython.build.base import Base, shell_command


class BuildSystem(Base):
    """
    autopkgtest test runner. All the methods that are not related to
    testing (configure, build, clean etc) are not implemented by this class, on
    purpose.
    """

    NAME = "autopkgtest"
    DESCRIPTION = 'autopkgtest test runner'
    SUPPORTED_INTERPRETERS = {'python3', 'python{version}'}

    @shell_command
    def test(self, context, args):
        return super().test(context, args)
